@@ -114,8 +114,8 @@ namespace exec {
         template <__decays_to<__t> _Self, receiver _Receiver>
           requires sender_to<__copy_cvref_t<_Self, _Constrained>, _Receiver>
         [[nodiscard]]
-        static auto connect(_Self&& __self, _Receiver __rcvr) //
-          -> __when_empty_op_t<_Self, _Receiver> {
+        static auto
+          connect(_Self&& __self, _Receiver __rcvr) -> __when_empty_op_t<_Self, _Receiver> {
           return __when_empty_op_t<_Self, _Receiver>{
             __self.__scope_, static_cast<_Self&&>(__self).__c_, static_cast<_Receiver&&>(__rcvr)};
         }
@@ -127,7 +127,7 @@ namespace exec {
         }
 
         const __impl* __scope_;
-        STDEXEC_ATTRIBUTE((no_unique_address)) _Constrained __c_;
+        STDEXEC_ATTRIBUTE(no_unique_address) _Constrained __c_;
       };
     };
 
@@ -140,7 +140,7 @@ namespace exec {
     struct __nest_op_base : __immovable {
       using _Receiver = stdexec::__t<_ReceiverId>;
       const __impl* __scope_;
-      STDEXEC_ATTRIBUTE((no_unique_address)) _Receiver __rcvr_;
+      STDEXEC_ATTRIBUTE(no_unique_address) _Receiver __rcvr_;
     };
 
     template <class _ReceiverId>
@@ -243,7 +243,7 @@ namespace exec {
         using sender_concept = stdexec::sender_t;
 
         const __impl* __scope_;
-        STDEXEC_ATTRIBUTE((no_unique_address)) _Constrained __c_;
+        STDEXEC_ATTRIBUTE(no_unique_address) _Constrained __c_;
 
         template <class _Receiver>
         using __nest_operation_t =
@@ -313,7 +313,7 @@ namespace exec {
           typename stop_token_of_t<env_of_t<_Receiver>>::template callback_type<__forward_stopped>;
 
         void __complete_() noexcept {
-          try {
+          STDEXEC_TRY {
             __forward_consumer_.reset();
             auto __state = std::move(__state_);
             STDEXEC_ASSERT(__state != nullptr);
@@ -346,15 +346,17 @@ namespace exec {
                 },
                 __state->__data_);
             }
-          } catch (...) {
+          }
+          STDEXEC_CATCH_ALL {
 
             stdexec::set_error(static_cast<_Receiver&&>(__rcvr_), std::current_exception());
           }
         }
 
-        STDEXEC_ATTRIBUTE((no_unique_address)) _Receiver __rcvr_;
+        STDEXEC_ATTRIBUTE(no_unique_address) _Receiver __rcvr_;
         std::unique_ptr<__future_state<_Sender, _Env>> __state_;
-        STDEXEC_ATTRIBUTE((no_unique_address)) stdexec::__optional<__forward_consumer> __forward_consumer_;
+        STDEXEC_ATTRIBUTE(no_unique_address)
+        stdexec::__optional<__forward_consumer> __forward_consumer_;
 
        public:
         using __id = __future_op;
@@ -369,8 +371,8 @@ namespace exec {
               return;
             }
             __raw_state->__no_future_ = std::move(__state_);
-            __raw_state->__step_from_to_(
-              __guard, __future_step::__future, __future_step::__no_future);
+            __raw_state
+              ->__step_from_to_(__guard, __future_step::__future, __future_step::__no_future);
           }
         }
 
@@ -388,7 +390,7 @@ namespace exec {
         }
 
         void start() & noexcept {
-          try {
+          STDEXEC_TRY {
             if (!!__state_) {
               std::unique_lock __guard{__state_->__mutex_};
               if (__state_->__data_.index() != 0) {
@@ -398,7 +400,8 @@ namespace exec {
                 __state_->__subscribers_.push_back(this);
               }
             }
-          } catch (...) {
+          }
+          STDEXEC_CATCH_ALL {
             stdexec::set_error(static_cast<_Receiver&&>(__rcvr_), std::current_exception());
           }
         }
@@ -422,8 +425,8 @@ namespace exec {
     auto __completion_as_tuple_(_Tag (*)(_Ts...)) -> std::tuple<_Tag, _Ts...>;
 
     template <class _Fn>
-    using __completion_as_tuple_t =
-      decltype(__scope::__completion_as_tuple_(static_cast<_Fn*>(nullptr)));
+    using __completion_as_tuple_t = decltype(__scope::__completion_as_tuple_(
+      static_cast<_Fn*>(nullptr)));
 #endif
 
     template <class... _Ts>
@@ -433,19 +436,19 @@ namespace exec {
     using __decay_error_t = completion_signatures<set_error_t(__decay_t<_Ty>)>;
 
     template <class _Sender, class _Env>
-    using __future_completions_t = //
-      transform_completion_signatures_of<
-        _Sender,
-        __env_t<_Env>,
-        completion_signatures<set_stopped_t(), set_error_t(std::exception_ptr)>,
-        __decay_values_t,
-        __decay_error_t>;
+    using __future_completions_t = transform_completion_signatures_of<
+      _Sender,
+      __env_t<_Env>,
+      completion_signatures<set_stopped_t(), set_error_t(std::exception_ptr)>,
+      __decay_values_t,
+      __decay_error_t
+    >;
 
     template <class _Completions>
-    using __completions_as_variant = //
-      __mapply<
-        __mtransform<__q<__completion_as_tuple_t>, __mbind_front_q<std::variant, std::monostate>>,
-        _Completions>;
+    using __completions_as_variant = __mapply<
+      __mtransform<__q<__completion_as_tuple_t>, __mbind_front_q<std::variant, std::monostate>>,
+      _Completions
+    >;
 
     template <class _Ty>
     struct __dynamic_delete {
@@ -544,10 +547,11 @@ namespace exec {
         template <class _Tag, class... _As>
         void __save_completion(_Tag, _As&&... __as) noexcept {
           auto& __state = *__state_;
-          try {
+          STDEXEC_TRY {
             using _Tuple = __decayed_std_tuple<_Tag, _As...>;
             __state.__data_.template emplace<_Tuple>(_Tag(), static_cast<_As&&>(__as)...);
-          } catch (...) {
+          }
+          STDEXEC_CATCH_ALL {
             using _Tuple = std::tuple<set_error_t, std::exception_ptr>;
             __state.__data_.template emplace<_Tuple>(set_error_t(), std::current_exception());
           }
@@ -590,15 +594,26 @@ namespace exec {
     struct __future_state : __future_state_base<__future_completions_t<_Sender, _Env>, _Env> {
       using _Completions = __future_completions_t<_Sender, _Env>;
 
-      __future_state(_Sender __sndr, _Env __env, const __impl* __scope)
+      __future_state(connect_t, _Sender&& __sndr, _Env __env, const __impl* __scope)
         : __future_state_base<_Completions, _Env>(static_cast<_Env&&>(__env), __scope)
-        , __op_(
-            stdexec::connect(
-              static_cast<_Sender&&>(__sndr),
-              __future_receiver_t<_Sender, _Env>{this, __scope})) {
+        , __op_(static_cast<_Sender&&>(__sndr), __future_receiver_t<_Sender, _Env>{this, __scope}) {
       }
 
-      connect_result_t<_Sender, __future_receiver_t<_Sender, _Env>> __op_;
+      __future_state(_Sender __sndr, _Env __env, const __impl* __scope)
+        : __future_state(
+            stdexec::connect,
+            static_cast<_Sender&&>(__sndr),
+            static_cast<_Env&&>(__env),
+            __scope) {
+        // If the operation completes synchronously, then the following line will cause
+        // the destruction of *this, which is not a problem because we used a delegating
+        // constructor, so *this is considered fully constructed.
+        __op_.submit(
+          static_cast<_Sender&&>(__sndr), __future_receiver_t<_Sender, _Env>{this, __scope});
+      }
+
+      STDEXEC_ATTRIBUTE(no_unique_address)
+      submit_result<_Sender, __future_receiver_t<_Sender, _Env>> __op_{};
     };
 
     template <class _SenderId, class _EnvId>
@@ -631,8 +646,8 @@ namespace exec {
               return;
             }
             __raw_state->__no_future_ = std::move(__state_);
-            __raw_state->__step_from_to_(
-              __guard, __future_step::__future, __future_step::__no_future);
+            __raw_state
+              ->__step_from_to_(__guard, __future_step::__future, __future_step::__no_future);
           }
         }
 
@@ -670,17 +685,19 @@ namespace exec {
     struct __spawn_env_ {
       inplace_stop_token __token_;
 
+      [[nodiscard]]
       auto query(get_stop_token_t) const noexcept -> inplace_stop_token {
         return __token_;
       }
 
-      auto query(get_scheduler_t) const noexcept -> __inln::__scheduler {
+      [[nodiscard]]
+      auto query(get_scheduler_t) const noexcept -> stdexec::inline_scheduler {
         return {};
       }
     };
 
     template <class _Env>
-    using __spawn_env_t = __env::__join_t<_Env, __spawn_env_>;
+    using __spawn_env_t = __join_env_t<_Env, __spawn_env_>;
 
     template <class _EnvId>
     struct __spawn_op_base {
@@ -727,21 +744,30 @@ namespace exec {
       using _Sender = stdexec::__t<_SenderId>;
 
       struct __t : __spawn_op_base<_EnvId> {
-        template <__decays_to<_Sender> _Sndr>
-        __t(_Sndr&& __sndr, _Env __env, const __impl* __scope)
-          : __spawn_op_base<_EnvId>{__env::__join(static_cast<_Env&&>(__env),
-            __spawn_env_{__scope->__stop_source_.get_token()}),
-            [](__spawn_op_base<_EnvId>* __op) {
-                delete static_cast<__t*>(__op);
-            }}
-          , __op_(stdexec::connect(static_cast<_Sndr&&>(__sndr), __spawn_receiver_t<_Env>{this})) {
+        __t(connect_t, _Sender&& __sndr, _Env __env, const __impl* __scope)
+          : __spawn_op_base<
+              _EnvId
+            >{__env::__join(
+                static_cast<_Env&&>(__env),
+                __spawn_env_{__scope->__stop_source_.get_token()}),
+              [](__spawn_op_base<_EnvId>* __op) { delete static_cast<__t*>(__op); }}
+          , __data_(static_cast<_Sender&&>(__sndr), __spawn_receiver_t<_Env>{this}) {
         }
 
-        void start() & noexcept {
-          stdexec::start(__op_);
+        __t(_Sender __sndr, _Env __env, const __impl* __scope)
+          : __t(
+              stdexec::connect,
+              static_cast<_Sender&&>(__sndr),
+              static_cast<_Env&&>(__env),
+              __scope) {
+          // If the operation completes synchronously, then the following line will cause
+          // the destruction of *this, which is not a problem because we used a delegating
+          // constructor, so *this is considered fully constructed.
+          __data_.submit(static_cast<_Sender&&>(__sndr), __spawn_receiver_t<_Env>{this});
         }
 
-        connect_result_t<_Sender, __spawn_receiver_t<_Env>> __op_;
+        STDEXEC_ATTRIBUTE(no_unique_address)
+        submit_result<_Sender, __spawn_receiver_t<_Env>> __data_;
       };
     };
 
@@ -773,23 +799,22 @@ namespace exec {
         return nest_result_t<_Constrained>{&__impl_, static_cast<_Constrained&&>(__c)};
       }
 
-      template <__movable_value _Env = empty_env, sender_in<__spawn_env_t<_Env>> _Sender>
+      template <__movable_value _Env = env<>, sender_in<__spawn_env_t<_Env>> _Sender>
         requires sender_to<nest_result_t<_Sender>, __spawn_receiver_t<_Env>>
       void spawn(_Sender&& __sndr, _Env __env = {}) {
         using __op_t = __spawn_operation_t<nest_result_t<_Sender>, _Env>;
-        // start is noexcept so we can assume that the operation will complete
-        // after this, which means we can rely on its self-ownership to ensure
-        // that it is eventually deleted
-        stdexec::start(*(
-          new __op_t{nest(static_cast<_Sender&&>(__sndr)), static_cast<_Env&&>(__env), &__impl_}));
+        // this will connect and start the operation, after which the operation state is
+        // responsible for deleting itself after it completes.
+        [[maybe_unused]]
+        auto* __op =
+          new __op_t{nest(static_cast<_Sender&&>(__sndr)), static_cast<_Env&&>(__env), &__impl_};
       }
 
-      template <__movable_value _Env = empty_env, sender_in<__env_t<_Env>> _Sender>
+      template <__movable_value _Env = env<>, sender_in<__env_t<_Env>> _Sender>
       auto spawn_future(_Sender&& __sndr, _Env __env = {}) -> __future_t<_Sender, _Env> {
         using __state_t = __future_state<nest_result_t<_Sender>, _Env>;
         auto __state = std::make_unique<__state_t>(
           nest(static_cast<_Sender&&>(__sndr)), static_cast<_Env&&>(__env), &__impl_);
-        stdexec::start(__state->__op_);
         return __future_t<_Sender, _Env>{std::move(__state)};
       }
 
@@ -813,6 +838,6 @@ namespace exec {
   using __scope::async_scope;
 
   template <class _AsyncScope, class _Sender>
-  using nest_result_t =
-    decltype(stdexec::__declval<_AsyncScope&>().nest(stdexec::__declval<_Sender&&>()));
+  using nest_result_t = decltype(stdexec::__declval<_AsyncScope&>()
+                                   .nest(stdexec::__declval<_Sender&&>()));
 } // namespace exec
