@@ -20,15 +20,15 @@
 
 // TODO: This is needed for libdispatch to compile with GCC. Need to look for
 // workaround.
-#ifndef __has_feature
-#  define __has_feature(x) false
-#endif
-#ifndef __has_extension
-#  define __has_extension(x) false
-#endif
+#  ifndef __has_feature
+#    define __has_feature(x) false
+#  endif
+#  ifndef __has_extension
+#    define __has_extension(x) false
+#  endif
 
-#include "stdexec/execution.hpp"
-#include <dispatch/dispatch.h>
+#  include "../stdexec/execution.hpp"
+#  include <dispatch/dispatch.h>
 
 namespace exec {
   struct libdispatch_queue;
@@ -67,10 +67,10 @@ namespace exec {
 
     template <class Fun, class Shape, class... Args>
       requires stdexec::__callable<Fun, Shape, Args &...>
-    using bulk_non_throwing = //
-      stdexec::__mbool<
-        stdexec::__nothrow_callable<Fun, Shape, Args &...>
-        && noexcept(stdexec::__decayed_std_tuple<Args...>(std::declval<Args>()...))>;
+    using bulk_non_throwing = stdexec::__mbool<
+      stdexec::__nothrow_callable<Fun, Shape, Args &...>
+      && noexcept(stdexec::__decayed_std_tuple<Args...>(std::declval<Args>()...))
+    >;
 
     template <class CvrefSenderId, class ReceiverId, class Shape, class Fun, bool MayThrow>
     struct bulk_receiver {
@@ -81,7 +81,8 @@ namespace exec {
 
     template <class CvrefSender, class Receiver, class Shape, class Fun, bool MayThrow>
     using bulk_receiver_t = stdexec::__t<
-      bulk_receiver<stdexec::__cvref_id<CvrefSender>, stdexec::__id<Receiver>, Shape, Fun, MayThrow>>;
+      bulk_receiver<stdexec::__cvref_id<CvrefSender>, stdexec::__id<Receiver>, Shape, Fun, MayThrow>
+    >;
 
     template <class CvrefSenderId, class ReceiverId, std::integral Shape, class Fun>
     struct bulk_op_state {
@@ -95,12 +96,14 @@ namespace exec {
       stdexec::__id<stdexec::__decay_t<Sender>>,
       stdexec::__id<stdexec::__decay_t<Receiver>>,
       Shape,
-      Fun>>;
+      Fun
+    >>;
 
     struct transform_bulk {
       template <class Data, class Sender>
       auto operator()(stdexec::bulk_t, Data &&data, Sender &&sndr) {
-        auto [shape, fun] = std::forward<Data>(data);
+        auto [pol, shape, fun] = std::forward<Data>(data);
+        // TODO: handle non-par execution policies
         return bulk_sender_t<Sender, decltype(shape), decltype(fun)>{
           queue_, std::forward<Sender>(sndr), shape, std::move(fun)};
       }
@@ -119,16 +122,15 @@ namespace exec {
       template <stdexec::sender_expr_for<stdexec::bulk_t> Sender>
       auto transform_sender(Sender &&sndr) const noexcept {
         if constexpr (stdexec::__completes_on<Sender, libdispatch_scheduler>) {
-          auto sched =
-            stdexec::get_completion_scheduler<stdexec::set_value_t>(stdexec::get_env(sndr));
+          auto sched = stdexec::get_completion_scheduler<stdexec::set_value_t>(
+            stdexec::get_env(sndr));
           return stdexec::__sexpr_apply(
             std::forward<Sender>(sndr), __libdispatch_bulk::transform_bulk{*sched.queue_});
         } else {
           static_assert(
             stdexec::__completes_on<Sender, libdispatch_scheduler>,
-            "No libdispatch_queue instance can be found in the "
-            "sender's environment "
-            "on which to schedule bulk work.");
+            "No libdispatch_queue instance can be found in the sender's "
+            "attributes on which to schedule bulk work.");
           return __libdispatch_details::not_a_sender<stdexec::__name_of<Sender>>();
         }
       }
@@ -136,21 +138,14 @@ namespace exec {
       // transform the generic bulk sender into a parallel libdispatch bulk sender
       template <stdexec::sender_expr_for<stdexec::bulk_t> Sender, class Env>
       auto transform_sender(Sender &&sndr, const Env &env) const noexcept {
-        if constexpr (stdexec::__completes_on<Sender, libdispatch_scheduler>) {
-          auto sched =
-            stdexec::get_completion_scheduler<stdexec::set_value_t>(stdexec::get_env(sndr));
-          return stdexec::__sexpr_apply(
-            std::forward<Sender>(sndr), __libdispatch_bulk::transform_bulk{*sched.queue_});
-        } else if constexpr (stdexec::__starts_on<Sender, libdispatch_scheduler, Env>) {
+        if constexpr (stdexec::__starts_on<Sender, libdispatch_scheduler, Env>) {
           auto sched = stdexec::get_scheduler(env);
           return stdexec::__sexpr_apply(
             std::forward<Sender>(sndr), __libdispatch_bulk::transform_bulk{*sched.queue_});
         } else {
-          static_assert( //
-            stdexec::__starts_on<Sender, libdispatch_scheduler, Env>
-              || stdexec::__completes_on<Sender, libdispatch_scheduler>,
-            "No libdispatch_queue instance can be found in the sender's or "
-            "receiver's "
+          static_assert(
+            stdexec::__starts_on<Sender, libdispatch_scheduler, Env>,
+            "No libdispatch_queue instance can be found in the receiver's "
             "environment on which to schedule bulk work.");
           return __libdispatch_details::not_a_sender<stdexec::__name_of<Sender>>();
         }
@@ -165,8 +160,7 @@ namespace exec {
         stdexec::completion_signatures<stdexec::set_value_t(), stdexec::set_stopped_t()>;
 
       template <stdexec::receiver Receiver>
-      auto connect(Receiver r) const //
-        -> __libdispatch_details::operation<stdexec::__id<Receiver>> {
+      auto connect(Receiver r) const -> __libdispatch_details::operation<stdexec::__id<Receiver>> {
         return __libdispatch_details::operation<stdexec::__id<Receiver>>(queue, std::move(r));
       }
 
@@ -264,41 +258,43 @@ namespace exec {
     Fun fun_;
 
     template <class Sender, class... Env>
-    using with_error_invoke_t = //
-      stdexec::__if_c<
-        stdexec::__v<stdexec::__value_types_t<
-          stdexec::__completion_signatures_of_t<Sender, Env...>,
-          stdexec::__mbind_front_q<bulk_non_throwing, Fun, Shape>,
-          stdexec::__q<stdexec::__mand>>>,
-        stdexec::completion_signatures<>,
-        stdexec::__eptr_completion>;
+    using with_error_invoke_t = stdexec::__if_c<
+      stdexec::__v<stdexec::__value_types_t<
+        stdexec::__completion_signatures_of_t<Sender, Env...>,
+        stdexec::__mbind_front_q<bulk_non_throwing, Fun, Shape>,
+        stdexec::__q<stdexec::__mand>
+      >>,
+      stdexec::completion_signatures<>,
+      stdexec::__eptr_completion
+    >;
 
     template <class... Tys>
     using set_value_t =
       stdexec::completion_signatures<stdexec::set_value_t(stdexec::__decay_t<Tys>...)>;
 
     template <class Self, class... Env>
-    using __completions_t = //
-      stdexec::transform_completion_signatures<
-        stdexec::__completion_signatures_of_t<stdexec::__copy_cvref_t<Self, Sender>, Env...>,
-        with_error_invoke_t<stdexec::__copy_cvref_t<Self, Sender>, Env...>,
-        set_value_t>;
+    using __completions_t = stdexec::transform_completion_signatures<
+      stdexec::__completion_signatures_of_t<stdexec::__copy_cvref_t<Self, Sender>, Env...>,
+      with_error_invoke_t<stdexec::__copy_cvref_t<Self, Sender>, Env...>,
+      set_value_t
+    >;
 
     template <class Self, class Receiver>
-    using bulk_op_state_t = //
-      stdexec::__t<
-        bulk_op_state<stdexec::__cvref_id<Self, Sender>, stdexec::__id<Receiver>, Shape, Fun>>;
+    using bulk_op_state_t = stdexec::__t<
+      bulk_op_state<stdexec::__cvref_id<Self, Sender>, stdexec::__id<Receiver>, Shape, Fun>
+    >;
 
     template <stdexec::__decays_to<__t> Self, stdexec::receiver Receiver>
       requires stdexec::receiver_of<Receiver, __completions_t<Self, stdexec::env_of_t<Receiver>>>
-    static bulk_op_state_t<Self, Receiver> connect(Self &&self, Receiver rcvr) //
-      noexcept(stdexec::__nothrow_constructible_from<
-               bulk_op_state_t<Self, Receiver>,
-               libdispatch_queue &,
-               Shape,
-               Fun,
-               Sender,
-               Receiver>) {
+    static bulk_op_state_t<Self, Receiver>
+      connect(Self &&self, Receiver rcvr) noexcept(stdexec::__nothrow_constructible_from<
+                                                   bulk_op_state_t<Self, Receiver>,
+                                                   libdispatch_queue &,
+                                                   Shape,
+                                                   Fun,
+                                                   Sender,
+                                                   Receiver
+      >) {
       return bulk_op_state_t<Self, Receiver>{
         self.queue_,
         self.shape_,
@@ -340,9 +336,10 @@ namespace exec {
           };
 
           if constexpr (MayThrow) {
-            try {
+            STDEXEC_TRY {
               sh_state.apply(computation);
-            } catch (...) {
+            }
+            STDEXEC_CATCH_ALL {
               std::uint32_t expected = total_tasks;
 
               if (sh_state.task_with_exception_.compare_exchange_strong(
@@ -376,12 +373,12 @@ namespace exec {
       }
     };
 
-    using variant_t = //
-      stdexec::__value_types_of_t<
-        CvrefSender,
-        stdexec::env_of_t<Receiver>,
-        stdexec::__q<stdexec::__decayed_std_tuple>,
-        stdexec::__q<stdexec::__std_variant>>;
+    using variant_t = stdexec::__value_types_of_t<
+      CvrefSender,
+      stdexec::env_of_t<Receiver>,
+      stdexec::__q<stdexec::__decayed_std_tuple>,
+      stdexec::__q<stdexec::__std_variant>
+    >;
 
     variant_t data_;
     Receiver rcvr_;
@@ -436,9 +433,10 @@ namespace exec {
       using tuple_t = stdexec::__decayed_std_tuple<As...>;
 
       if constexpr (MayThrow) {
-        try {
+        STDEXEC_TRY {
           shared_state_.data_.template emplace<tuple_t>(std::move(as)...);
-        } catch (...) {
+        }
+        STDEXEC_CATCH_ALL {
           stdexec::set_error(std::move(shared_state_.rcvr_), std::current_exception());
         }
       } else {
@@ -472,12 +470,12 @@ namespace exec {
   struct __libdispatch_bulk::bulk_op_state<CvrefSenderId, ReceiverId, Shape, Fun>::__t {
     using __id = bulk_op_state;
 
-    static constexpr bool may_throw = //
-      !stdexec::__v<stdexec::__value_types_of_t<
-        CvrefSender,
-        stdexec::env_of_t<Receiver>,
-        stdexec::__mbind_front_q<bulk_non_throwing, Fun, Shape>,
-        stdexec::__q<stdexec::__mand>>>;
+    static constexpr bool may_throw = !stdexec::__v<stdexec::__value_types_of_t<
+      CvrefSender,
+      stdexec::env_of_t<Receiver>,
+      stdexec::__mbind_front_q<bulk_non_throwing, Fun, Shape>,
+      stdexec::__q<stdexec::__mand>
+    >>;
 
     using bulk_rcvr = bulk_receiver_t<CvrefSender, Receiver, Shape, Fun, may_throw>;
     using shared_state = bulk_shared_state<CvrefSender, Receiver, Shape, Fun, may_throw>;
