@@ -150,9 +150,6 @@ _FIt2 copy(_FIt1 __first, _FIt1 __last, _FIt2 __d_first) {
   using _Index = typename std::iterator_traits<_FIt1>::difference_type;
   _Index __input_size = std::distance(__first, __last);
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::copy(__first, __last, __d_first);
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   #pragma acc_stdpar parallel loop
@@ -344,9 +341,6 @@ _FIt max_element(_FIt __first, _FIt __last, _BF __cmp) {
   using _Index = typename std::iterator_traits<_FIt>::difference_type;
   _Index __input_size = std::distance(__first, __last);
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::max_element(__first, __last, __cmp);
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _Index* __partial_max = new _Index[__num_chunks];
@@ -386,9 +380,6 @@ _FIt3 merge(_FIt1 __first1, _FIt1 __last1, _FIt2 __first2, _FIt2 __last2,
     return __d_first;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::merge(__first1, __last1, __first2, __last2, __d_first, __cmp);
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   #pragma acc_stdpar parallel loop
@@ -413,9 +404,6 @@ _FIt min_element(_FIt __first, _FIt __last, _BF __cmp) {
   using _Index = typename std::iterator_traits<_FIt>::difference_type;
   _Index __input_size = std::distance(__first, __last);
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::min_element(__first, __last, __cmp);
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _Index* __partial_min = new _Index[__num_chunks];
@@ -446,16 +434,13 @@ _FIt min_element(_FIt __first, _FIt __last, _BF __cmp) {
 
 template <class _FIt, class _BF>
 std::pair<_FIt, _FIt> minmax_element(_FIt __first, _FIt __last, _BF __cmp) {
-  if (__first == __last) {
-    return std::make_pair(__first, __first);
-  }
   using _Index = typename std::iterator_traits<_FIt>::difference_type;
   _Index __input_size = std::distance(__first, __last);
+  if (__input_size <= 1) {
+    return std::make_pair(__first, __first);
+  }
   _Index __num_pairs = __input_size / 2;
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::minmax_element(__first, __last, __cmp);
-  }
   _Index __chunk_size = __num_pairs / __num_chunks;
   _Index __leftover = __num_pairs % __num_chunks;
   // If the input size is odd, the last element is its own chunk.
@@ -645,6 +630,36 @@ _FIt reverse_copy(_BDIt __first, _BDIt __last, _FIt __d_first) {
   return __d_first + __input_size;
 }
 
+//========== rotate ==========
+
+template <class _FIt>
+_FIt rotate(_FIt __first, _FIt __n_first, _FIt __last) {
+  using _Index = typename std::iterator_traits<_FIt>::difference_type;
+  if (__first == __n_first) {
+    return __last;
+  }
+  if (__n_first == __last) {
+    return __first;
+  }
+  _Index __input_size_half = std::distance(__first, __last) / 2;
+  _Index __front_size_half = std::distance(__first, __n_first) / 2;
+  _Index __back_size_half = std::distance(__n_first, __last) / 2;
+  #pragma acc_stdpar parallel loop
+  for (_Index __i = 0; __i < __front_size_half + __back_size_half; ++__i) {
+    if (__i < __front_size_half) {
+      std::iter_swap(__first + __i, __n_first - (__i + 1));
+    } else {
+      _Index __j = __i - __front_size_half;
+      std::iter_swap(__n_first + __j, __last - (__j + 1));
+    }
+  }
+  #pragma acc_stdpar parallel loop
+  for (_Index __i = 0; __i < __input_size_half; ++__i) {
+    std::iter_swap(__first + __i, __last - (__i + 1));
+  }
+  return __first + std::distance(__n_first, __last);
+}
+
 //========== rotate_copy ==========
 
 template <class _FIt1, class _FIt2>
@@ -707,6 +722,73 @@ _FIt1 search(_FIt1 __first, _FIt1 __last, _FIt2 __s_first, _FIt2 __s_last,
   return __last;
 }
 
+//========== shift_left ==========
+
+template <class _FIt>
+_FIt shift_left(_FIt __first, _FIt __last,
+                typename std::iterator_traits<_FIt>::difference_type __n) {
+  using _Index = typename std::iterator_traits<_FIt>::difference_type;
+  if (__n <= 0) {
+    return __last;
+  }
+  _Index __input_size = std::distance(__first, __last);
+  if (__n >= __input_size) {
+    return __first;
+  }
+  if (__n < 50000 && __n < __input_size / 128) {
+    // Too much overlap for parallel move to be effective.  Use a
+    // double-reverse instead, where each reverse is parallelized.
+    __openacc::reverse(__first + __n, __last);
+    __openacc::reverse(__first, __last);
+  } else {
+    // Use a parallel move assignment.
+    _Index __shifted_size = __input_size - __n;
+    _Index __threads = __detail::min(__n, __shifted_size);
+    #pragma acc_stdpar parallel loop
+    for (_Index __i = 0; __i < __threads; ++__i) {
+      #pragma acc_stdpar loop seq
+      for (_Index __j = __i; __j < __shifted_size; __j += __n) {
+        __first[__j] = std::move(__first[__j + __n]);
+      }
+    }
+  }
+  return __first + (__input_size - __n);
+}
+
+//========== shift_right ==========
+
+template <class _FIt>
+_FIt shift_right(_FIt __first, _FIt __last,
+                 typename std::iterator_traits<_FIt>::difference_type __n) {
+  using _Index = typename std::iterator_traits<_FIt>::difference_type;
+  if (__n <= 0) {
+    return __first;
+  }
+  _Index __input_size = std::distance(__first, __last);
+  if (__n >= __input_size) {
+    return __last;
+  }
+  if (__n < 50000 && __n < __input_size / 128) {
+    // Too much overloap for parallel move to be effective.  Use a
+    // double-reverse instead, where each reverse is parallelized.
+    __openacc::reverse(__first, __last - __n);
+    __openacc::reverse(__first, __last);
+  } else {
+    // Use a parallel move assignment.
+    _Index __shifted_size = __input_size - __n;
+    _Index __threads = __detail::min(__n, __shifted_size);
+    _FIt __last_elem = __last - 1;
+    #pragma acc_stdpar parallel loop
+    for (_Index __i = 0; __i < __threads; ++__i) {
+      #pragma acc_stdpar loop seq
+      for (_Index __j = __i; __j < __shifted_size; __j += __n) {
+        __last_elem[-__j] = std::move(__last_elem[-(__j + __n)]);
+      }
+    }
+  }
+  return __first + __n;
+}
+
 //========== stable_sort ==========
 
 template <class _RIt, class _BF>
