@@ -416,139 +416,179 @@ struct __op_is_instance_of
           __logical_or<std::is_same<typename std::decay<_Op>::type,
                                     _OpTmpl<_ArgTypes>>::value...>::value> { };
 
-// The OpenACC back end.  Only some algorithms are implemented, and those are
-// implemented only for some conditions.
+}}
+
+#if __NVCOMPILER_STDPAR_OPENACC_GPU || _NVHPC_STDPAR_GPU
+#include <nvhpc/numeric_openacc.hpp>
+#endif
+
+namespace std { namespace __stdpar {
+
+// The OpenACC back end.  Only some algorithms are implemented.
 
 template <> struct __numeric_impl<__back_end::__openacc> {
 
 #if __NVCOMPILER_STDPAR_OPENACC_GPU || _NVHPC_STDPAR_GPU
 
-  // Whenever an iterator is a raw pointer, use a 'deviceptr' clause so that
-  // the compiler doesn't implicitly manage the data referenced by that
-  // pointer.
+  //========== exclusive_scan ==========
+
+#if _NVHPC_STDPAR_ACC_EXPERIMENTAL
+  template <class _FIt1, class _FIt2, class _T>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 exclusive_scan(__no_policy, _FIt1 __first, _FIt1 __last,
+                              _FIt2 __d_first, _T __init) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    return __openacc::exclusive_scan(__first, __last, __d_first, __init,
+                                     std::plus<_T>());
+  }
+
+  template <class _FIt1, class _FIt2, class _T, class _BF>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 exclusive_scan(__no_policy, _FIt1 __first, _FIt1 __last,
+                              _FIt2 __d_first, _T __init, _BF __f) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    _ASSERT_NOT_FUNC_PTR(_BF);
+    return __openacc::exclusive_scan(__first, __last, __d_first, __init, __f);
+  }
+#endif
+
+  //========== inclusive_scan ==========
+
+#if _NVHPC_STDPAR_ACC_EXPERIMENTAL
+  template <class _FIt1, class _FIt2>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 inclusive_scan(__no_policy, _FIt1 __first, _FIt1 __last,
+                              _FIt2 __d_first) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    return __openacc::inclusive_scan(
+      __first, __last, __d_first,
+      std::plus<typename std::iterator_traits<_FIt1>::value_type>());
+  }
+
+  template <class _FIt1, class _FIt2, class _BF>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 inclusive_scan(__no_policy, _FIt1 __first, _FIt1 __last,
+                              _FIt2 __d_first, _BF __f) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    _ASSERT_NOT_FUNC_PTR(_BF);
+    return __openacc::inclusive_scan(__first, __last, __d_first, __f);
+  }
+
+  template <class _FIt1, class _FIt2, class _BF, class _T>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 inclusive_scan(__no_policy, _FIt1 __first, _FIt1 __last,
+                              _FIt2 __d_first, _BF __f, _T __init) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    _ASSERT_NOT_FUNC_PTR(_BF);
+    return __openacc::inclusive_scan(__first, __last, __d_first, __f, __init);
+  }
+#endif
+
   //========== reduce ==========
 
   template <class _FIt, class _T>
   _NVHPC_PARALLEL_IMPL_OPENACC
-  static typename std::enable_if<std::is_scalar<_T>::value, _T>::type
-  reduce(__no_policy, _FIt __first, _FIt __last, _T __init) {
-    return reduce(__no_policy{}, __first, __last, __init, std::plus<_T>{});
+  static _T reduce(__no_policy, _FIt __first, _FIt __last, _T __init) {
+    _ASSERT_RANDOM_ACCESS(_FIt);
+    return __openacc::reduce(__first, __last, __init, std::plus<_T>{});
   }
 
   template <class _FIt, class _T, class _BF>
-  _NVHPC_PARALLEL_IMPL_OPENACC static
-  typename std::enable_if<std::is_scalar<_T>::value &&
-                            __op_is_instance_of<_BF, std::plus, _T>::value,
-                          _T>::type
-  reduce(__no_policy, _FIt __first, _FIt __last, _T __init, _BF __f) {
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _T reduce(__no_policy, _FIt __first, _FIt __last, _T __init, _BF __f) {
     _ASSERT_RANDOM_ACCESS(_FIt);
     _ASSERT_NOT_FUNC_PTR(_BF);
-    _T __result = __init;
-    auto __input_size = std::distance(__first, __last);
-    if constexpr (!std::is_pointer<_FIt>::value) {
-      _Pragma("acc_stdpar parallel loop reduction(+:__result)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + *(__first + __i);
-      }
-    } else {
-      static_assert(std::is_pointer<_FIt>::value,
-                    "internal error: unhandled OpenACC parallel loop variant");
-      _Pragma(
-        "acc_stdpar parallel loop reduction(+:__result) deviceptr(__first)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + *(__first + __i);
-      }
-    }
-    return __result;
+    return __openacc::reduce(__first, __last, __init, __f);
   }
 
+  //========== transform_exclusive_scan ==========
+
+#if _NVHPC_STDPAR_ACC_EXPERIMENTAL
+  template <class _FIt1, class _FIt2, class _T, class _BF, class _UF>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 transform_exclusive_scan(__no_policy, _FIt1 __first,
+                                        _FIt1 __last, _FIt2 __d_first,
+                                        _T __init, _BF __fsum, _UF __ft) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    _ASSERT_NOT_FUNC_PTR(_BF);
+    _ASSERT_NOT_FUNC_PTR(_UF);
+    return __openacc::transform_exclusive_scan(__first, __last, __d_first,
+                                               __init, __fsum, __ft);
+  }
+#endif
+
+  //========== transform_inclusive_scan ==========
+
+#if _NVHPC_STDPAR_ACC_EXPERIMENTAL
+  template <class _FIt1, class _FIt2, class _BF, class _UF>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 transform_inclusive_scan(__no_policy, _FIt1 __first,
+                                        _FIt1 __last, _FIt2 __d_first,
+                                        _BF __fsum, _UF __ft) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    _ASSERT_NOT_FUNC_PTR(_BF);
+    _ASSERT_NOT_FUNC_PTR(_UF);
+    return __openacc::transform_inclusive_scan(__first, __last, __d_first,
+                                               __fsum, __ft);
+  }
+
+  template <class _FIt1, class _FIt2, class _BF, class _UF, class _T>
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _FIt2 transform_inclusive_scan(__no_policy, _FIt1 __first,
+                                        _FIt1 __last, _FIt2 __d_first,
+                                        _BF __fsum, _UF __ft, _T __init) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    _ASSERT_NOT_FUNC_PTR(_BF);
+    _ASSERT_NOT_FUNC_PTR(_UF);
+    return __openacc::transform_inclusive_scan(__first, __last, __d_first,
+                                               __fsum, __ft, __init);
+  }
+#endif
+
   //========== transform_reduce ==========
 
   template <class _FIt1, class _FIt2, class _T>
   _NVHPC_PARALLEL_IMPL_OPENACC
-  static typename std::enable_if<std::is_scalar<_T>::value, _T>::type
-  transform_reduce(__no_policy, _FIt1 __first1, _FIt1 __last1, _FIt2 __first2,
-                   _T __init) {
-    return transform_reduce(
-      __no_policy{}, __first1, __last1, __first2, __init, std::plus<_T>{},
-      [](typename std::iterator_traits<_FIt1>::reference __a,
-         typename std::iterator_traits<_FIt2>::reference __b) {
-        return __a * __b;
-      });
+  static _T transform_reduce(__no_policy, _FIt1 __first1, _FIt1 __last1,
+                             _FIt2 __first2, _T __init) {
+    _ASSERT_RANDOM_ACCESS(_FIt1);
+    _ASSERT_RANDOM_ACCESS(_FIt2);
+    return __openacc::transform_reduce(
+      __first1, __last1, __first2, __init, std::plus<_T>{},
+      std::multiplies<typename std::common_type<
+        typename std::iterator_traits<_FIt1>::value_type,
+        typename std::iterator_traits<_FIt2>::value_type>::type>{});
   }
 
   template <class _FIt1, class _FIt2, class _T, class _BF1, class _BF2>
-  _NVHPC_PARALLEL_IMPL_OPENACC static
-  typename std::enable_if<std::is_scalar<_T>::value &&
-                            __op_is_instance_of<_BF1, std::plus, _T>::value,
-                          _T>::type
-  transform_reduce(__no_policy, _FIt1 __first1, _FIt1 __last1, _FIt2 __first2,
-                   _T __init, _BF1 __f1, _BF2 __f2) {
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _T transform_reduce(__no_policy, _FIt1 __first1, _FIt1 __last1,
+                             _FIt2 __first2, _T __init, _BF1 __fsum,
+                             _BF2 __ft) {
     _ASSERT_RANDOM_ACCESS(_FIt1);
     _ASSERT_RANDOM_ACCESS(_FIt2);
     _ASSERT_NOT_FUNC_PTR(_BF1);
     _ASSERT_NOT_FUNC_PTR(_BF2);
-    _T __result = __init;
-    auto __input_size = std::distance(__first1, __last1);
-    if constexpr (!std::is_pointer<_FIt1>::value &&
-                  !std::is_pointer<_FIt2>::value) {
-      _Pragma("acc_stdpar parallel loop reduction(+:__result)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + __f2(*(__first1 + __i), *(__first2 + __i));
-      }
-    } else if constexpr (std::is_pointer<_FIt1>::value &&
-                         !std::is_pointer<_FIt2>::value) {
-      _Pragma(
-        "acc_stdpar parallel loop reduction(+:__result) deviceptr(__first1)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + __f2(*(__first1 + __i), *(__first2 + __i));
-      }
-    } else if constexpr (!std::is_pointer<_FIt1>::value &&
-                         std::is_pointer<_FIt2>::value) {
-      _Pragma(
-        "acc_stdpar parallel loop reduction(+:__result) deviceptr(__first2)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + __f2(*(__first1 + __i), *(__first2 + __i));
-      }
-    } else {
-      static_assert(std::is_pointer<_FIt1>::value &&
-                      std::is_pointer<_FIt2>::value,
-                    "internal error: unhandled OpenACC parallel loop variant");
-      _Pragma("acc_stdpar parallel loop reduction(+:__result) deviceptr(__first1, __first2)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + __f2(*(__first1 + __i), *(__first2 + __i));
-      }
-    }
-    return __result;
+    return __openacc::transform_reduce(__first1, __last1, __first2, __init,
+                                       __fsum, __ft);
   }
 
   template <class _FIt, class _T, class _BF, class _UF>
-  _NVHPC_PARALLEL_IMPL_OPENACC static
-  typename std::enable_if<std::is_scalar<_T>::value &&
-                            __op_is_instance_of<_BF, std::plus, _T>::value,
-                          _T>::type
-  transform_reduce(__no_policy, _FIt __first, _FIt __last, _T __init, _BF __f1,
-                   _UF __f2) {
+  _NVHPC_PARALLEL_IMPL_OPENACC
+  static _T transform_reduce(__no_policy, _FIt __first, _FIt __last, _T __init,
+                             _BF __fsum, _UF __ft) {
     _ASSERT_RANDOM_ACCESS(_FIt);
     _ASSERT_NOT_FUNC_PTR(_BF);
     _ASSERT_NOT_FUNC_PTR(_UF);
-    _T __result = __init;
-    auto __input_size = std::distance(__first, __last);
-    if constexpr (!std::is_pointer<_FIt>::value) {
-      _Pragma("acc_stdpar parallel loop reduction(+:__result)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + __f2(*(__first + __i));
-      }
-    } else {
-      static_assert(std::is_pointer<_FIt>::value,
-                    "internal error: unhandled OpenACC parallel loop variant");
-      _Pragma(
-        "acc_stdpar parallel loop reduction(+:__result) deviceptr(__first)")
-      for (decltype(__input_size) __i = 0; __i < __input_size; ++__i) {
-        __result = __result + __f2(*(__first + __i));
-      }
-    }
-    return __result;
+    return __openacc::transform_reduce(__first, __last, __init, __fsum, __ft);
   }
 
 #endif // __NVCOMPILER_STDPAR_OPENACC_GPU || _NVHPC_STDPAR_GPU
