@@ -29,9 +29,6 @@ _FIt2 inclusive_scan(_FIt1 __first, _FIt1 __last, _FIt2 __d_first, _BF __f) {
     return __d_first;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::partial_sum(__first, __last, __d_first, __f);
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _ValT* __partial_scan = new _ValT[__num_chunks];
@@ -62,7 +59,7 @@ _FIt2 inclusive_scan(_FIt1 __first, _FIt1 __last, _FIt2 __d_first, _BF __f) {
     _Index __this_size = __chunk_end - __chunk_start;
     #pragma acc_stdpar loop vector
     for (_Index __j = 0; __j < __this_size; ++__j) {
-      _Index __idx = __chunk_size + __j;
+      _Index __idx = __chunk_start + __j;
       __d_first[__idx] =
           __f(__partial_scan[__i - 1], std::move(__d_first[__idx]));
     }
@@ -81,15 +78,6 @@ _FIt2 inclusive_scan(_FIt1 __first, _FIt1 __last, _FIt2 __d_first, _BF __f,
     return __d_first;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    while (__first != __last) {
-      __init = __f(std::move(__init), *__first);
-      *__d_first = __init;
-      ++__first;
-      ++__d_first;
-    }
-    return __d_first;
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _ValT* __partial_scan = new _ValT[__num_chunks];
@@ -234,16 +222,6 @@ _FIt2 transform_exclusive_scan(_FIt1 __first, _FIt1 __last, _FIt2 __d_first,
     return __d_first;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    while (__first != __last) {
-      auto __new_sum = __fsum(__init, __ft(*__first));
-      *__d_first = std::move(__init);
-      __init = std::move(__new_sum);
-      ++__first;
-      ++__d_first;
-    }
-    return __d_first;
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _ValT* __partial_scan = new _ValT[__num_chunks];
@@ -281,7 +259,7 @@ _FIt2 transform_exclusive_scan(_FIt1 __first, _FIt1 __last, _FIt2 __d_first,
     _Index __chunk_start =
         __detail::__chunk_start(__i, __chunk_size, __leftover);
     _Index __chunk_end = __detail::__chunk_end(__i, __chunk_size, __leftover);
-    _Index __this_size = __chunk_end - __chunk_size;
+    _Index __this_size = __chunk_end - __chunk_start;
     #pragma acc_stdpar loop vector
     for (_Index __j = 0; __j < __this_size; ++__j) {
       _Index __idx = __chunk_start + __j;
@@ -305,19 +283,6 @@ _FIt2 transform_inclusive_scan(_FIt1 __first, _FIt1 __last, _FIt2 __d_first,
     return __d_first;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    _ValT __sum = __ft(*__first);
-    *__d_first = __sum;
-    ++__first;
-    ++__d_first;
-    while (__first != __last) {
-      __sum = __fsum(std::move(__sum), __ft(*__first));
-      *__d_first = __sum;
-      ++__first;
-      ++__d_first;
-    }
-    return __d_first;
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _ValT* __partial_scan = new _ValT[__num_chunks];
@@ -367,15 +332,6 @@ _FIt2 transform_inclusive_scan(_FIt1 __first, _FIt1 __last, _FIt2 __d_first,
     return __d_first;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    while (__first != __last) {
-      __init = __fsum(std::move(__init), __ft(*__first));
-      *__d_first = __init;
-      ++__first;
-      ++__d_first;
-    }
-    return __d_first;
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _ValT* __partial_scan = new _ValT[__num_chunks];
@@ -497,10 +453,6 @@ transform_reduce(_FIt1 __first1, _FIt1 __last1, _FIt2 __first2, _T __init,
     return __init;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::inner_product(__first1, __last1, __first2, __init, __fsum,
-                              __ft);
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _T* __partial_sums = new _T[__num_chunks];
@@ -535,14 +487,6 @@ transform_reduce(_FIt __first, _FIt __last, _T __init, _BF __fsum, _UF __ft) {
     return __init;
   }
   _Index __num_chunks = __detail::__iterations_for_reduce_or_scan(__input_size);
-  if (__num_chunks == 0) {
-    return std::inner_product(
-        __first, __last, __first, __init, __fsum,
-        [=](typename std::iterator_traits<_FIt>::value_type __x,
-            typename std::iterator_traits<_FIt>::value_type __y) mutable {
-          return __ft(__x);
-        });
-  }
   _Index __chunk_size = __input_size / __num_chunks;
   _Index __leftover = __input_size % __num_chunks;
   _T* __partial_sums = new _T[__num_chunks];
