@@ -473,12 +473,12 @@ inline int __get_device_total_thread_count() {
 
 template <class _Index>
 _Index __iterations_for_reduce_or_scan(_Index __input_size) {
-  if (__input_size < 1024) {
-    return 0;
-  }
   if (__input_size < (1 << 22)) {
-    return __get_device_sm_count() *
-        std::lround(0.0614 * std::sqrt(__input_size));
+    return __detail::max(
+        1,
+        __detail::min(__input_size,
+                      __get_device_sm_count() *
+                          std::lround(0.0614 * std::sqrt(__input_size))));
   }
   return (__get_device_sm_count() * 128) *
       std::lround(0.95 * std::sqrt(__input_size) / (8192) + 0.95);
