@@ -221,6 +221,15 @@ enum, bind(c)
   enumerator cudaMemAdviseUnsetAccessedBy
 end enum
 
+enum, bind(c)
+  enumerator :: cudaMemLocationTypeInvalid = 0
+  enumerator cudaMemLocationTypeNone = 0
+  enumerator cudaMemLocationTypeDevice
+  enumerator cudaMemLocationTypeHost
+  enumerator cudaMemLocationTypeHostNuma
+  enumerator cudaMemLocationTypeHostNumaCurrent
+end enum
+
 enum, bind(c)
   enumerator :: cudaMemRangeAttributeReadMostly = 1
   enumerator cudaMemRangeAttributePreferredLocation
@@ -366,17 +375,19 @@ enum, bind(c)
   enumerator :: cudaDevAttrPageableMemoryAccessUsesHostPageTables = 100
   enumerator :: cudaDevAttrDirectManagedMemAccessFromHost   = 101
   enumerator :: cudaDevAttrMaxBlocksPerMultiprocessor       = 106
+  enumerator :: cudaDevAttrMaxPersistingL2CacheSize         = 108
+  enumerator :: cudaDevAttrMaxAccessPolicyWindowSize        = 109
   enumerator :: cudaDevAttrReservedSharedMemoryPerBlock     = 111
   enumerator :: cudaDevAttrSparseCudaArraySupported         = 112
   enumerator :: cudaDevAttrHostRegisterReadOnlySupported    = 113
-  enumerator :: cudaDevAttrMaxTimelineSemaphoreInteropSupported   = 114
+  enumerator :: cudaDevAttrMaxTimelineSemaphoreInteropSupported = 114
   enumerator :: cudaDevAttrMemoryPoolsSupported             = 115
   enumerator :: cudaDevAttrGPUDirectRDMASupported           = 116
   enumerator :: cudaDevAttrGPUDirectRDMAFlushWritesOptions  = 117
   enumerator :: cudaDevAttrGPUDirectRDMAWritesOrdering      = 118
   enumerator :: cudaDevAttrMemoryPoolSupportedHandleTypes   = 119
   enumerator :: cudaDevAttrClusterLaunch                    = 120
-  enumerator :: cudaDevAttrDeferredMappingCudaArraySupported      = 121
+  enumerator :: cudaDevAttrDeferredMappingCudaArraySupported    = 121
   enumerator :: cudaDevAttrReserved122                      = 122
   enumerator :: cudaDevAttrReserved123                      = 123
   enumerator :: cudaDevAttrReserved124                      = 124
@@ -388,8 +399,18 @@ enum, bind(c)
   enumerator :: cudaDevAttrNumaConfig                       = 130
   enumerator :: cudaDevAttrNumaId                           = 131
   enumerator :: cudaDevAttrReserved132                      = 132
-  enumerator :: cudaDevAttrReserved133                      = 133
+  enumerator :: cudaDevAttrMpsEnabled                       = 133
   enumerator :: cudaDevAttrHostNumaId                       = 134
+  enumerator :: cudaDevAttrD3D12CigSupported                = 135
+  enumerator :: cudaDevAttrVulkanCigSupported               = 138
+  enumerator :: cudaDevAttrGpuPciDeviceId                   = 139
+  enumerator :: cudaDevAttrGpuPciSubsystemId                = 140
+  enumerator :: cudaDevAttrReserved141                      = 141
+  enumerator :: cudaDevAttrHostNumaMemoryPoolsSupported     = 142
+  enumerator :: cudaDevAttrHostNumaMultinodeIpcSupported    = 143
+  enumerator :: cudaDevAttrHostMemoryPoolsSupported         = 144
+  enumerator :: cudaDevAttrReserved145                      = 145
+  enumerator :: cudaDevAttrOnlyPartialHostNativeAtomicSupported = 147
   enumerator cudaDevAttrMax
 end enum
 
